import { Set } from 'immutable';
/**
 * Split the decorations in lists of relevant decorations for each child.
 *
 * @param {Node} node
 * @param {List} decorations
 * @return {Array<List<Decoration>>}
 */
function getChildrenDecorations(node, decorations) {
    var activeDecorations = Set().asMutable();
    var childrenDecorations = [];
    orderChildDecorations(node, decorations).forEach(function (item) {
        if (item.isRangeStart) {
            // Item is a decoration start
            activeDecorations.add(item.decoration);
        }
        else if (item.isRangeEnd) {
            // item is a decoration end
            activeDecorations.remove(item.decoration);
        }
        else {
            // Item is a child node
            childrenDecorations.push(activeDecorations.toList());
        }
    });
    return childrenDecorations;
}
/**
 * Orders the children of provided node and its decoration endpoints (start, end)
 * so that decorations can be passed only to relevant children (see use in Node.render())
 *
 * @param {Node} node
 * @param {List} decorations
 * @return {Array<Item>}
 *
 * where type Item =
 * {
 *   child: Node,
 *   // Index of the child in its parent
 *   index: number
 * }
 * or {
 *   // True if this represents the start of the given decoration
 *   isRangeStart: boolean,
 *   // True if this represents the end of the given decoration
 *   isRangeEnd: boolean,
 *   decoration: Range
 * }
 */
function orderChildDecorations(node, decorations) {
    var _a;
    if (decorations.isEmpty()) {
        return node.nodes.toArray().map(function (child, index) { return ({
            child: child,
            index: index,
        }); });
    }
    // Map each key to its global order
    var keyOrders = (_a = {}, _a[node.key] = 0, _a);
    var globalOrder = 1;
    node.forEachDescendant(function (child) {
        keyOrders[child.key] = globalOrder;
        globalOrder = globalOrder + 1;
    });
    var childNodes = node.nodes.toArray();
    var endPoints = childNodes.map(function (child, index) { return ({
        child: child,
        index: index,
        order: keyOrders[child.key],
    }); });
    decorations.forEach(function (decoration) {
        // Range start.
        // A rangeStart should be before the child containing its startKey, in order
        // to consider it active before going down the child.
        var startKeyOrder = keyOrders[decoration.start.key];
        var containingChildOrder = startKeyOrder === undefined
            ? 0
            : getContainingChildOrder(childNodes, keyOrders, startKeyOrder);
        endPoints.push({
            isRangeStart: true,
            order: containingChildOrder - 0.5,
            decoration: decoration,
        });
        // Range end.
        var endKeyOrder = (keyOrders[decoration.end.key] || globalOrder) + 0.5;
        endPoints.push({
            isRangeEnd: true,
            order: endKeyOrder,
            decoration: decoration,
        });
    });
    return endPoints.sort(function (a, b) { return (a.order > b.order ? 1 : -1); });
}
/*
 * Returns the key order of the child right before the given order.
 */
function getContainingChildOrder(children, keyOrders, order) {
    // Find the first child that is after the given key
    var nextChildIndex = children.findIndex(function (child) { return order < keyOrders[child.key]; });
    if (nextChildIndex <= 0) {
        return 0;
    }
    var containingChild = children[nextChildIndex - 1];
    return keyOrders[containingChild.key];
}
/**
 * Export.
 *
 * @type {Function}
 */
export default getChildrenDecorations;
