"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.default = void 0;

var _debug = _interopRequireDefault(require("debug"));

var _getWindow = _interopRequireDefault(require("get-window"));

var _reactDom = require("react-dom");

var _slateHotkeys = _interopRequireDefault(require("@zykj/slate-hotkeys"));

var _slateDevEnvironment = require("@zykj/slate-dev-environment");

var _findNode = _interopRequireDefault(require("../utils/find-node"));

/**
 * Debug.
 *
 * @type {Function}
 */
var debug = (0, _debug.default)("slate:before");
/**
 * The core before plugin.
 *
 * @return {Object}
 */

function BeforePlugin() {
  var activeElement = null;
  var compositionCount = 0;
  var isComposing = false;
  var isCopying = false;
  var isDragging = false;
  /**
   * On before input.
   *
   * @param {Event} event
   * @param {Change} change
   * @param {Editor} editor
   */

  function onBeforeInput(event, change, editor) {
    if (editor.props.readOnly) return true;
    var isSynthetic = !!event.nativeEvent; // COMPAT: If the browser supports Input Events Level 2, we will have
    // attached a custom handler for the real `beforeinput` events, instead of
    // allowing React's synthetic polyfill, so we need to ignore synthetics.

    if (isSynthetic && _slateDevEnvironment.HAS_INPUT_EVENTS_LEVEL_2) return true;
    debug("onBeforeInput", {
      event: event
    });
  }
  /**
   * On blur.
   *
   * @param {Event} event
   * @param {Change} change
   * @param {Editor} editor
   */


  function onBlur(event, change, editor) {
    if (isCopying) return true;
    if (editor.props.readOnly) return true;
    var value = change.value;
    var schema = value.schema;
    var relatedTarget = event.relatedTarget,
        target = event.target;
    var window = (0, _getWindow.default)(target); // COMPAT: If the current `activeElement` is still the previous one, this is
    // due to the window being blurred when the tab itself becomes unfocused, so
    // we want to abort early to allow to editor to stay focused when the tab
    // becomes focused again.

    if (activeElement == window.document.activeElement) return true; // COMPAT: The `relatedTarget` can be null when the new focus target is not
    // a "focusable" element (eg. a `<div>` without `tabindex` set).

    if (relatedTarget) {
      var el = (0, _reactDom.findDOMNode)(editor); // COMPAT: The event should be ignored if the focus is returning to the
      // editor from an embedded editable element (eg. an <input> element inside
      // a void node).

      if (relatedTarget == el) return true; // COMPAT: The event should be ignored if the focus is moving from the
      // editor to inside a void node's spacer element.

      if (relatedTarget.hasAttribute("data-slate-spacer")) return true; // COMPAT: The event should be ignored if the focus is moving to a non-
      // editable section of an element that isn't a void node (eg. a list item
      // of the check list example).

      var node = (0, _findNode.default)(relatedTarget, value);
      if (el.contains(relatedTarget) && node && !schema.isVoid(node)) return true;
    }

    debug("onBlur", {
      event: event
    });
  }
  /**
   * On change.
   *
   * @param {Change} change
   * @param {Editor} editor
   */


  function onChange(change, editor) {
    var value = change.value; // If the value's schema isn't the editor's schema, update it. This can
    // happen on the initialization of the editor, or if the schema changes.
    // This change isn't save into history since only schema is updated.

    if (value.schema != editor.schema) {
      change.setValue({
        schema: editor.schema
      }, {
        save: false
      }).normalize();
    }

    debug("onChange");
  }
  /**
   * On composition end.
   *
   * @param {Event} event
   * @param {Change} change
   * @param {Editor} editor
   */


  function onCompositionEnd(event, change, editor) {
    var n = compositionCount; // The `count` check here ensures that if another composition starts
    // before the timeout has closed out this one, we will abort unsetting the
    // `isComposing` flag, since a composition is still in affect.

    window.requestAnimationFrame(function () {
      if (compositionCount > n) return;
      isComposing = false; // HACK: we need to re-render the editor here so that it will update its
      // placeholder in case one is currently rendered. This should be handled
      // differently ideally, in a less invasive way?
      // (apply force re-render if isComposing changes)

      if (editor.state.isComposing) {
        editor.setState({
          isComposing: false
        });
      }
    });
    debug("onCompositionEnd", {
      event: event
    });
  }
  /**
   * On composition start.
   *
   * @param {Event} event
   * @param {Change} change
   * @param {Editor} editor
   */


  function onCompositionStart(event, change, editor) {
    if (change.value.selection.isExpanded) {
      change.delete();
    }

    debug("onCompositionStart", {
      event: event
    });
  }
  /**
   * on composition update
   * @param event
   * @param change
   * @param editor
   */


  function onCompositionUpdate(event, change, editor) {
    isComposing = true;
    compositionCount++; // HACK: we need to re-render the editor here so that it will update its
    // placeholder in case one is currently rendered. This should be handled
    // differently ideally, in a less invasive way?
    // (apply force re-render if isComposing changes)

    if (!editor.state.isComposing) {
      editor.setState({
        isComposing: true
      });
    }

    debug("onCompositionUpdate", {
      event: event
    });
  }
  /**
   * On copy.
   *
   * @param {Event} event
   * @param {Change} change
   * @param {Editor} editor
   */


  function onCopy(event, change, editor) {
    var window = (0, _getWindow.default)(event.target);
    isCopying = true;
    window.requestAnimationFrame(function () {
      return isCopying = false;
    });
    debug("onCopy", {
      event: event
    });
  }
  /**
   * On cut.
   *
   * @param {Event} event
   * @param {Change} change
   * @param {Editor} editor
   */


  function onCut(event, change, editor) {
    if (editor.props.readOnly) return true;
    var window = (0, _getWindow.default)(event.target);
    isCopying = true;
    window.requestAnimationFrame(function () {
      return isCopying = false;
    });
    debug("onCut", {
      event: event
    });
  }
  /**
   * On drag end.
   *
   * @param {Event} event
   * @param {Change} change
   * @param {Editor} editor
   */


  function onDragEnd(event, change, editor) {
    isDragging = false;
    debug("onDragEnd", {
      event: event
    });
  }
  /**
   * On drag enter.
   *
   * @param {Event} event
   * @param {Change} change
   * @param {Editor} editor
   */


  function onDragEnter(event, change, editor) {
    debug("onDragEnter", {
      event: event
    });
  }
  /**
   * On drag exit.
   *
   * @param {Event} event
   * @param {Change} change
   * @param {Editor} editor
   */


  function onDragExit(event, change, editor) {
    debug("onDragExit", {
      event: event
    });
  }
  /**
   * On drag leave.
   *
   * @param {Event} event
   * @param {Change} change
   * @param {Editor} editor
   */


  function onDragLeave(event, change, editor) {
    debug("onDragLeave", {
      event: event
    });
  }
  /**
   * On drag over.
   *
   * @param {Event} event
   * @param {Change} change
   * @param {Editor} editor
   */


  function onDragOver(event, change, editor) {
    // If the target is inside a void node, and only in this case,
    // call `preventDefault` to signal that drops are allowed.
    // When the target is editable, dropping is already allowed by
    // default, and calling `preventDefault` hides the cursor.
    var value = editor.value;
    var schema = value.schema;
    var node = (0, _findNode.default)(event.target, editor.value);
    if (schema.isVoid(node)) event.preventDefault(); // COMPAT: IE won't call onDrop on contentEditables unless the
    // default dragOver is prevented:
    // https://developer.microsoft.com/en-us/microsoft-edge/platform/issues/913982/
    // (2018/07/11)

    if (_slateDevEnvironment.IS_IE) event.preventDefault(); // If a drag is already in progress, don't do this again.

    if (!isDragging) {
      isDragging = true; // COMPAT: IE will raise an `unspecified error` if dropEffect is
      // set. (2018/07/11)

      if (!_slateDevEnvironment.IS_IE) {
        event.nativeEvent.dataTransfer.dropEffect = "move";
      }
    }

    debug("onDragOver", {
      event: event
    });
  }
  /**
   * On drag start.
   *
   * @param {Event} event
   * @param {Change} change
   * @param {Editor} editor
   */


  function onDragStart(event, change, editor) {
    isDragging = true;
    debug("onDragStart", {
      event: event
    });
  }
  /**
   * On drop.
   *
   * @param {Event} event
   * @param {Change} change
   * @param {Editor} editor
   */


  function onDrop(event, change, editor) {
    // Nothing happens in read-only mode.
    if (editor.props.readOnly) return true; // Prevent default so the DOM's value isn't corrupted.

    event.preventDefault();
    debug("onDrop", {
      event: event
    });
  }
  /**
   * On focus.
   *
   * @param {Event} event
   * @param {Change} change
   * @param {Editor} editor
   */


  function onFocus(event, change, editor) {
    if (isCopying) return true;
    if (editor.props.readOnly) return true;
    var el = (0, _reactDom.findDOMNode)(editor); // Save the new `activeElement`.

    var window = (0, _getWindow.default)(event.target);
    activeElement = window.document.activeElement; // COMPAT: If the editor has nested editable elements, the focus can go to
    // those elements. In Firefox, this must be prevented because it results in
    // issues with keyboard navigation. (2017/03/30)

    if (_slateDevEnvironment.IS_FIREFOX && event.target != el) {
      el.focus();
      return true;
    }

    debug("onFocus", {
      event: event
    });
  }
  /**
   * On input.
   *
   * @param {Event} event
   * @param {Change} change
   * @param {Editor} editor
   */


  function onInput(event, change, editor) {
    if (isComposing) return true;
    if (change.value.selection.isBlurred) return true;
    debug("onInput", {
      event: event
    });
  }
  /**
   * On key down.
   *
   * @param {Event} event
   * @param {Change} change
   * @param {Editor} editor
   */


  function onKeyDown(event, change, editor) {
    if (editor.props.readOnly) return true; // When composing, we need to prevent all hotkeys from executing while
    // typing. However, certain characters also move the selection before
    // we're able to handle it, so prevent their default behavior.

    if (isComposing) {
      if (_slateHotkeys.default.isCompose(event)) event.preventDefault();
      return true;
    } // Certain hotkeys have native editing behaviors in `contenteditable`
    // elements which will change the DOM and cause our value to be out of sync,
    // so they need to always be prevented.


    if (!_slateDevEnvironment.IS_IOS && (_slateHotkeys.default.isBold(event) || _slateHotkeys.default.isDeleteBackward(event) || _slateHotkeys.default.isDeleteForward(event) || _slateHotkeys.default.isDeleteLineBackward(event) || _slateHotkeys.default.isDeleteLineForward(event) || _slateHotkeys.default.isDeleteWordBackward(event) || _slateHotkeys.default.isDeleteWordForward(event) || _slateHotkeys.default.isItalic(event) || _slateHotkeys.default.isRedo(event) || _slateHotkeys.default.isSplitBlock(event) || _slateHotkeys.default.isTransposeCharacter(event) || _slateHotkeys.default.isUndo(event))) {
      event.preventDefault();
    }

    debug("onKeyDown", {
      event: event
    });
  }
  /**
   * On paste.
   *
   * @param {Event} event
   * @param {Change} change
   * @param {Editor} editor
   */


  function onPaste(event, change, editor) {
    if (editor.props.readOnly) return true; // Prevent defaults so the DOM state isn't corrupted.

    event.preventDefault();
    debug("onPaste", {
      event: event
    });
  }
  /**
   * On select.
   *
   * @param {Event} event
   * @param {Change} change
   * @param {Editor} editor
   */


  function onSelect(event, change, editor) {
    if (isCopying) return true;
    if (isComposing) return true;
    if (editor.props.readOnly) return true; // Save the new `activeElement`.

    var window = (0, _getWindow.default)(event.target);
    activeElement = window.document.activeElement;
    debug("onSelect", {
      event: event
    });
  }
  /**
   * Return the plugin.
   *
   * @type {Object}
   */


  return {
    onBeforeInput: onBeforeInput,
    onBlur: onBlur,
    onChange: onChange,
    onCompositionEnd: onCompositionEnd,
    onCompositionStart: onCompositionStart,
    onCompositionUpdate: onCompositionUpdate,
    onCopy: onCopy,
    onCut: onCut,
    onDragEnd: onDragEnd,
    onDragEnter: onDragEnter,
    onDragExit: onDragExit,
    onDragLeave: onDragLeave,
    onDragOver: onDragOver,
    onDragStart: onDragStart,
    onDrop: onDrop,
    onFocus: onFocus,
    onInput: onInput,
    onKeyDown: onKeyDown,
    onPaste: onPaste,
    onSelect: onSelect
  };
}
/**
 * Export.
 *
 * @type {Object}
 */


var _default = BeforePlugin;
exports.default = _default;